'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SSR_DEPENDENCIES_MOCK = undefined;

var _reactRouterRedux = require('react-router-redux');

var _rxjs = require('rxjs');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SSR_DEPENDENCIES_MOCK = exports.SSR_DEPENDENCIES_MOCK = {
  notFound: function notFound() {},
  redirect: function redirect() {},
  observe: function observe(action, observable) {
    return observable;
  }
};

var SSR = function SSR(currentLocation) {
  var _this = this;

  _classCallCheck(this, SSR);

  this.onLoad = function (onLoad) {
    _this.onLoadCallback = onLoad;
    return _this;
  };

  this.onRedirect = function (onRedirect) {
    _this.onRedirectCallback = onRedirect;
    return _this;
  };

  this.onNotFound = function (onNotFound) {
    _this.onNotFoundCallback = onNotFound;
    return _this;
  };

  this.dispatchInitialAction = function (store) {
    _this._setStore(store);
    if (_this._hasValidStore()) {
      _this.store.dispatch(_this.initialAction);
    }
  };

  this.middleware = function () {
    return function (store) {
      return function (next) {
        return function (action) {
          // eslint-disable-line

          if (action.type.indexOf('SSR/') === 0 && !_this.loadingComplete) _this._onUpdate(action);

          next(action);
        };
      };
    };
  };

  this.observe = function (action, reduxObservable) {

    var time = new Date().getTime();
    _this._addPendingAction({
      actionType: action.type,
      time: time
    });

    var obs = _rxjs.Observable.zip(reduxObservable, _rxjs.Observable.of({
      type: SSR.ACTION_TYPES.SUCCESS,
      actionType: action.type,
      time: time
    }));
    return obs.flatMap(function (actions) {
      return actions;
    });
  };

  this.redirect = function (redirectUrl) {
    _this._onLoadingComplete();
    _this.onRedirectCallback(_this.store, {
      status: 301,
      redirectUrl: redirectUrl
    });
  };

  this.notFound = function () {
    _this._onLoadingComplete();
    _this.onNotFoundCallback(_this.store, {
      status: 404
    });
  };

  this._onLoadingComplete = function () {
    _this.loadingComplete = true;
    Object.keys(SSR_DEPENDENCIES_MOCK).forEach(function (mockKey) {
      _this[mockKey] = SSR_DEPENDENCIES_MOCK[mockKey];
    });
  };

  this._setStore = function (store) {
    _this.store = store;
  };

  this._hasValidStore = function () {
    if (!_this.store) throw new Error('SSR: store must be set after initialization!!!');
    return true;
  };

  this._addPendingAction = function (ssrAction) {
    _this.pendingActions.push(ssrAction);
    _this.store.dispatch(Object.assign({}, ssrAction, {
      type: SSR.ACTION_TYPES.PENDING
    }));
  };

  this._onUpdate = function (action) {
    switch (action.type) {
      case SSR.ACTION_TYPES.SUCCESS:
        {
          _this.pendingActions = _this.pendingActions.filter(function (pendingAction) {
            return action.time !== pendingAction.time && action.actionType !== pendingAction.actionType;
          });
          break;
        }
      case SSR.ACTION_TYPES.PENDING:
        {
          break;
        }
      case SSR.ACTION_TYPES.ERROR:
        {
          break;
        }
    }
    if (_this.pendingActions.length === 0 && !_this.loadingComplete) {
      _this._onLoadingComplete();
      _this.onLoadCallback(_this.store);
    }
  };

  this.pendingActions = [];
  this.store = null;
  this.initialAction = {
    type: _reactRouterRedux.LOCATION_CHANGE,
    payload: {
      pathname: currentLocation
    }
  };
  this.loadingComplete = false;

  this.actionCounter = 0;
};

SSR.ACTION_TYPES = {
  PENDING: 'SSR/PENDING',
  ERROR: 'SSR/ERROR',
  SUCCESS: 'SSR/SUCCESS'
};
exports.default = SSR;